package statej;

import java.io.PrintWriter;

import statej.ast.*;

/**
 * <p>Title: Programming mini project 2 Java-- compiler</p>
 * <p>Description: A visitor that visits the AST for the program being compiled. Implements the Visitor
 * pattern (Gamma et al)</p>
 * <p>Copyright: Copyright (c) 2004 NIcholas Cameron</p>
 * <p>Company: </p>
 * @author Nicholas Cameron
 * @version 1.0
 */

public abstract class Visitor
{
    //output
    protected PrintWriter err;
    //keep track of the number of errors
    protected int errorCount = 0;
    protected int warningCount = 0;

    public Visitor(PrintWriter err)
    {
        this.err = err;
    }

    /**
     * returns the number of errors that occured during the visit
     * @return
     */
    public int getErrorCount()
    {
        return errorCount;
    }
    public int getWarningCount()
    {
        return warningCount;
    }

    //report and log an error
    protected void error(String msg, Node node)
    {
        err.println("  Error: " + msg + " '" + node.toString() + "' on line " + node.getLineNumber() + ", column " + node.getColumnNumber());
        ++errorCount;
    }
    
    protected void warning(String msg, Node node)
    {
        err.println("  Warning: " + msg + " '" + node.toString() + "' on line " + node.getLineNumber() + ", column " + node.getColumnNumber());
        ++warningCount;
    }

    public abstract Object visit(AddNode node);
    public abstract Object visit(AndNode node);
    public abstract Object visit(ArgNode node);
    public abstract Object visit(AssNode node);
    public abstract Object visit(BracketNode node);
    public abstract Object visit(BooleanNode node);
    public abstract Object visit(ClassNode node);
    public abstract Object visit(CompoundNode node);
    public abstract Object visit(StateDefNode node);
    public abstract Object visit(ConstructorNode node);
    public abstract Object visit(DeclNode node);
    public abstract Object visit(DivNode node);
    public abstract Object visit(ComparNode node);
    public abstract Object visit(SpawnNode node);
    public abstract Object visit(FieldAccessNode node);
    public abstract Object visit(FieldNode node);
    public abstract Object visit(ForNode node);
    public abstract Object visit(IfNode node);
    public abstract Object visit(ListNode node);
    public abstract Object visit(MethodInvocNode node);
    public abstract Object visit(MethodNode node);
    public abstract Object visit(MultNode node);
    public abstract Object visit(NegNode node);
    public abstract Object visit(NewNode node);
    public abstract Object visit(NotNode node);
    public abstract Object visit(NullNode node);
    public abstract Object visit(NumberNode node);
    public abstract Object visit(OrNode node);
    public abstract Object visit(ImportNode node);
    public abstract Object visit(ReturnNode node);
    public abstract Object visit(StringNode node);
    public abstract Object visit(SubNode node);
    public abstract Object visit(SuperCallNode node);
    public abstract Object visit(VarNode node);
    public abstract Object visit(StateChangeNode node);
    public abstract Object visit(ProgramNode node);
    public abstract Object visit(PackageNode node);
}