package statej.ast;

import statej.Visitor;

import statej.typeChecker.ScopeSymbol;

/**
 * <p>Title: Programming mini project 2 Java-- compiler</p>
 * <p>Description: represents a compound statement</p>
 * <p>Copyright: Copyright (c) 2004 Nicholas Cameron</p>
 * <p>Company: </p>
 * @author Nicholas Cameron
 * @version 1.0
 */

public class CompoundNode extends Node
{
    private ListNode stmts;
    protected ExprNode synch;

    private ScopeSymbol symbol = null;

    public CompoundNode(ListNode stmts, ExprNode synch, int lineNumber, int colNumber)
    {
        super(lineNumber, colNumber);
        this.stmts = stmts;
        this.synch = synch;
    }

    public String toString()
    {
    	if (synch == null)
    		return "{\n" + stmts + "\n}";
    	else
    		return "synchronized (" + synch + ")\n{\n" + stmts + "\n}";
    }

    public Object accept(Visitor v)
    {
        return v.visit(this);
    }

    public Object visitSynch(Visitor v)
    {
    	if (synch == null)
    		return null;
    	else
    		return synch.accept(v);
    }

    public Object[] visitChildren(Visitor v)
    {
        if (stmts != null)
        {
            Node[] kids = stmts.toArray();
            Object[] result = new Object[kids.length];
            for (int i = 0; i < kids.length; ++i)
            {
            	result[i] = kids[i].accept(v);
            }
            
            return result;
        }
        else
        	return new Object[0];
    }

    public void setSymbol(ScopeSymbol symbol)
    {
        this.symbol = symbol;
    }
    public ScopeSymbol getSymbol()
    {
        return symbol;
    }
    
    public String eol()
    {
    	return "\n";
    }
    
    public String[] getEols()
    {
    	if (stmts == null)
    		return new String[0];
    	
    	Node[] elements = stmts.toArray();
    	String [] result = new String[elements.length];
    	
    	for (int i = 0; i < result.length; ++i)
    	{
    		result[i] = elements[i].eol();
    	}
    	
    	return result;
    }
}
