package statej.ast;

import statej.Visitor;


/**
 * <p>Title: Programming mini project 2 Java-- compiler</p>
 * <p>Description: An abstract node in the ast for the progam being compiled<br>
 * The comments in this class serve for all the node subclasses, it should be fairly
 * obvious what they all do. The ast implements the composite pattern and also the Visitor pattern</p>
 * <p>Copyright: Copyright (c) 2004 Nicholas Cameron</p>
 * <p>Company: </p>
 * @author Nicholas Cameron
 * @version 1.0
 */

public abstract class Node
{
    //the line and column number of the node's position in the source code
    protected int lineNumber, colNumber;

    /**
     * Construct a Node that was parsed at the given position in the source file
     * @param lineNumber the node's source's line number
     * @param colNumber the node's source's column number (the first char in the node's source)
     */
    protected Node(int lineNumber, int colNumber)
    {
        this.lineNumber = lineNumber;
        this.colNumber = colNumber;
    }

    /**
     * accept a visitor of the tree (see Visitor pattern, Gamma et al)
     * @param v the visitor to accept
     * @return some result of processing the node
     */
    public abstract Object accept(Visitor v);

    /**
     * returns the node's line number
     * @return the node's line number
     */
    public int getLineNumber()
    {
        return lineNumber;
    }

    /**
     * returns the node's column number
     * @return the node's column number
     */
    public int getColumnNumber()
    {
        return colNumber;
    }

    /**
     * returns the static type of the node, overridden by the sub-classes
     * @return "void" unless overridden by a sub-class
     */
    public String getType()
    {
        return "void";
    }
    
    /**
     * How an experssion should be terminated if it is promoted to a statement
     * throws a runtime exception if the expression is non-promotable
     * @return ";"
     */
    public String eol()
    {
    	return ";";
    }
}