package statej.utils;
import statej.ast.*;

import statej.typeChecker.Symbol;

/**
 * <p>Title: Programming mini project 2 Java-- compiler</p>
 * <p>Description: Provides a number of utility methods for formatting parameters</p>
 * <p>Copyright: Copyright (c) 2004 Nicholas Cameron</p>
 * <p>Company: </p>
 * @author Nicholas Cameron
 * @version 1.0
 */

public class ParamUtils
{
    private ParamUtils()
    {
    }

    /**
     * returns a list as a comma seperated String of the elements supplied
     * @param elements an array of Strings
     * @return a String eg "" or "eg" or "eg, eg"
     */
    public static String makeList(String[] elements)
    {
        String result =  "";
        for (int i = 0; i < elements.length; ++i)
        {
            result += elements[i] + ", ";
        }
        if (result.length() >= 2)
            result = result.substring(0, result.length() - 2);

        return result;
    }

    /**
     * returns a list as a comma seperated String of the types of
     * the elements supplied. Eg use: displaying the types of parameters of a
     * method symbol
     * @param elements an array of symbols
     * @return a String eg "" or "eg" or "eg, eg"
     */
    public static String makeTypeList(Symbol[] elements)
    {
        String result =  "";
        for (int i = 0; i < elements.length; ++i)
        {
            result += elements[i].getType() + ", ";
        }
        if (result.length() >= 2)
            result = result.substring(0, result.length() - 2);

        return result;
    }

    /**
     * returns a list as a comma seperated String of the types of
     * the elements in the ListNode supplied. Eg use: displaying the types of parameters of a
     * method call
     * @param elements a ListNode
     * @return a String eg "" or "eg" or "eg, eg"
     */
    public static String makeTypeList(ListNode elements)
    {
        if (elements == null)
            return "";

        Node[] elArray = elements.toArray();

        String result =  "";
        for (int i = 0; i < elArray.length; ++i)
        {
            result += elArray[i].getType() + ", ";
        }
        if (result.length() >= 2)
            result = result.substring(0, result.length() - 2);

        return result;
    }

    /**
     * returns a list as a comma seperated String of the values(as they appear in the source code) of
     * the elements in the ListNode supplied. Eg use: displaying the parameters of a
     * method call
     * @param elements a ListNode
     * @return a String eg "" or "eg" or "eg, eg"
     */
    public static String makeSourceList(ListNode elements)
    {
        if (elements == null)
            return "";

        Node[] elArray = elements.toArray();

        String result =  "";
        for (int i = 0; i < elArray.length; ++i)
        {
            result += elArray[i].toString() + ", ";
        }
        if (result.length() >= 2)
            result = result.substring(0, result.length() - 2);

        return result;
    }

    /**
     * makes an array of type names from a given array of types, found by visiting parameters
     * with a SemanticChecker
     * @param types
     * @return
     */
    public static String[] makeTypeArray(Object[] types)
    {
        String[] typeNames = new String[types.length];
        for (int i = 0; i < types.length; ++i)
            typeNames[i] = ((Symbol)types[i]).getName();

        return typeNames;
    }

    /**
     * replaces the character \n with the escape sequence for that character
     * ie \\n
     * @param input
     * @return
     */
    public static String escape(String input)
    {
        StringBuffer buf = new StringBuffer(input);
        int index = 0;
        while (index < buf.length())
        {
            if (buf.charAt(index) == '\n')
                buf.replace(index, ++index, "\\n");
            else
                ++index;
        }

        return buf.toString();
    }
}

